program test6;

(*********************************************************************
 *
 * This programs sets inputs and outputs of a U12 device
 * using the libljacklm library.
 *
 * kms
 * march 2016
 *
 *********************************************************************)

uses
  ctypes,
  SysUtils;

const
{$IFDEF MSWINDOWS}
  libljacklmName = 'libljacklm.dll';
{$ENDIF MSWINDOWS}
{$IFDEF LINUX}
  libljacklmName = 'libljacklm.so.0';
{$ENDIF LINUX}
{$IFDEF DARWIN}
  libljacklmName = 'libljacklm.dylib';
  {$linklib libljacklm}
{$ENDIF DARWIN}
 
const
  numChannels = 2;  // Using 2 channels

type
  tErrorString   = array[1 .. 50]  of char;
  t2Integers     = array[1 .. numChannels] of clong;
  tVoltages      = array[1 .. numChannels + 1] of single; // exact length does not work!!

var
  errorNumber, demo: clong;
  resetCounter, idnum: clong;
  analogOut0, analogOut1: real;
  overVoltage: clong;
  errorString: tErrorString;
  channel, gain, writeD, readD, state: clong;
  voltage: single;
  count, ms: cdouble;
  i: integer;

function GetDriverVersion: cfloat;
         cdecl; external libljacklmName;
         
function GetFirmwareVersion(var idnum: clong): cfloat;
         cdecl; external libljacklmName;

procedure GetErrorString (errorcode: clong;
                          var errorString: tErrorString);
          cdecl; external libljacklmName;
          
function EDigitalIn(var idnum: clong;
                    demo, channel, readD: clong;
                    var state: clong): clong;
         cdecl; external libljacklmName;

function EDigitalOut(var idnum: clong;
                     demo, channel, writeD, state: clong): clong;
         cdecl; external libljacklmName;

function EAnalogIn (var idnum: clong;
                    demo, channel, gain: clong;
                    var overvoltage: clong;
                    var voltage: single): clong;
         cdecl; external libljacklmName;
          
function EAnalogOut(var idnum: clong;
                    demo: clong;
                    analogOut0, analogOut1: single): clong;
         cdecl; external libljacklmName;

function ECount(var idnum: clong;
                demo, resetCounter: clong;
                var count: cdouble;
                var ms: cdouble): clong;
         cdecl; external libljacklmName;
                         
procedure handleError (errorNumber: integer; functionName: tErrorString);
begin
  if errorNumber <> 0 then
  begin
    GetErrorString(errorNumber, errorString);
    writeln (functionName, ' error ', errorNumber, ': ', errorString);
    exit;
  end;
end;

begin
  writeln ('+-----------------------------------------+');
  writeln ('| Set inputs and outputs of a U12 device. |');
  writeln ('+-----------------------------------------+');

  writeln('Library version: ', GetDriverVersion:7:4);

  idnum := -1; // -1 means take the first device  
  writeln('*** Getting firmware version does intialization and takes a moment. ***');
  writeln('U12 Firmware version: ', GetFirmwareVersion(idnum):7:4);

  // AOUpdate specific paramters
  demo    := 0;   // Normal operations
  analogOut0     := 2.0;  // Voltage for AO0
  analogOut1     := 3.5;  // Voltage for AO1
  
  writeln('Setting AO0 to ', analogOut0:5:3, ' V and AO1 to ', analogOut1:5:3, ' V');
  errorNumber := EAnalogOut(idnum, demo, analogOut0, analogOut1);
  handleError(errorNumber, 'EAnalogOut');

  overVoltage := 0;  // Returns if overvoltage was detected (>1)
  gain    := 0;   // Gains.  Does not matter in this case since we are
  voltage := 0.0;  // Returned voltage readings.  Pass an array of all zeros.

  writeln('Reading single-ended analog input channels AI0 and AI1:');
  channel := 0;
  errorNumber := EAnalogIn(idnum, demo, channel, gain, overVoltage, voltage);
  handleError(errorNumber, 'EAnalogIn');
  writeln('Single-ended AI', channel, ': ', voltage:5:3, ' V, overVoltage: ', overVoltage);
  channel := 1;
  errorNumber := EAnalogIn(idnum, demo, channel, gain, overVoltage, voltage);
  handleError(errorNumber, 'EAnalogIn');
  writeln('Single-ended AI', channel, ': ', voltage:5:3, ' V, overVoltage: ', overVoltage);
  
  writeln('Reading differential analog input channels AI0-AI1 (channel 8) and AI2-AI3 (channel 9):');
  for i := 8 to 9 do
  begin
    channel := i;
    errorNumber := EAnalogIn(idnum, demo, channel, gain, overVoltage, voltage);
    handleError(errorNumber, 'EAnalogIn' + intToStr(channel) + ' differential.');
    writeln('Differential AI', (channel-8)*2, '-AI', (channel-8)*2+1, ': ', voltage:5:3, ' V, overVoltage: ', overVoltage);
  end;
  
  writeln('Setting digital lines IO1 and D1 to output-low, and IO2 to output-high');  
  channel := 1;
  writeD := 0;
  state := 0;
  errorNumber := EDigitalOut(idnum, demo, channel, writeD, state);
  handleError(errorNumber, 'EDigitalOut (IO1)');

  channel := 1;
  writeD := 1;
  state := 0;
  errorNumber := EDigitalOut(idnum, demo, channel, writeD, state);
  handleError(errorNumber, 'EDigitalOut (D1)');

  channel := 2;
  writeD := 0;
  state := 1;
  errorNumber := EDigitalOut(idnum, demo, channel, writeD, state);
  handleError(errorNumber, 'EDigitalOut (IO2)');

  writeln('Reading some digital inputs:');
  channel := 0;
  readD := 0;
  errorNumber := EDigitalIn(idnum, demo, channel, readD, state);
  handleError(errorNumber, 'EDigitalIn (IO0)');
  writeln('IO', channel, ' state: ', state);

  channel := 0;
  readD := 1;
  errorNumber := EDigitalIn(idnum, demo, channel, readD, state);
  handleError(errorNumber, 'EDigitalIn (D0)');
  writeln('D', channel, '  state: ', state);

  writeln('Reading Counter:');
  writeln('Resetting count and waiting 2 seconds for new data.');

  resetCounter := 1;
  errorNumber := ECount(idnum, demo, resetCounter, count, ms);
  handleError(errorNumber, 'ECount (reset)');

  //Wait for 2 seconds for data
  sleep(2000);  // milliseconds

  writeln('Read Counter reading with 2 second timer.');
  resetCounter := 0;
  errorNumber := ECount(idnum, demo, resetCounter, count, ms);
  handleError(errorNumber, 'ECount (read)');

  writeln('Count: ', count:15:0, ' OS timer: ', ms:15:0,' ms');
  
  writeln ('+-----------------------------------------------+');
  writeln ('| Finished inputs and outputs  of a U12 device. |');
  writeln ('+-----------------------------------------------+');
end.